from storage import StorableObject

import emails

from quixote import get_request

import storage

from form import *
from misc import simplify

from formdata import FormData
from anonylink import AnonymityLink
from roles import Role
from categories import Category

widget_classes = {
    'string': StringWidget,
    'email': EmailWidget,
    'text': TextWidget,
    'bool': CheckboxWidget,
    'item': SingleSelectWidget,
    'file': FileWidget,
}

widget_extra_attributes = {
    'text': ('cols', 'rows'),
    'string': ('size',),
}


class ConsultationField:
    id = None
    label = None
    type = None
    required = True

    # for type == 'items'
    items = None
    show_as_radio = False

    # for type == 'text'
    cols = None
    rows = None
    pre = False

    # for type == 'string'
    size = None

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k.replace('-', '_'), v)



class Consultation(StorableObject):
    _names = 'consultations'

    name = None
    url_name = None
    fields = None
    receiver_id = None
    category_id = None
    roles = None

    def data_class(self):
        cls = FormData
        cls._names = 'consultation-%s' % self.url_name
        return cls

    def migrate(self):
        changed = False

        if self.__dict__.has_key('receiver'):
            self.receiver_id = self.__dict__['receiver']
            del self.__dict__['receiver']
            changed = True

        if self.__dict__.has_key('category'):
            self.category_id = self.__dict__['category']
            del self.__dict__['category']
            changed = True

        if not self.url_name:
            try:
                int(self.id)
            except ValueError:
                self.url_name = self.id
            else:
                self.url_name = simplify(self.name)
            changed = True

        if hasattr(self, 'questions'):
            for f in self.questions:
                if f.has_key('question'):
                    f['label'] = f['question']
                    del f['question']
                if f.has_key('answer'):
                    f['type'] = f['answer']
                    del f['answer']
            self.fields = [ConsultationField(**x) for x in self.questions]
            del self.questions
            for i, f in enumerate(self.fields):
                f.id = str(i)
            for formdata in self.data_class().select():
                for f in self.fields:
                    if not formdata.data.has_key(f.label):
                        continue
                    formdata.data[f.id] = formdata.data[f.label]
                    del formdata.data[f.label]
                formdata.store()
            changed = True

        if changed:
            self.store()

    def data_class(self):
        cls = FormData
        cls._names = 'consultation-%s' % self.url_name
        return cls


    def get_receiver(self):
        if self.receiver_id:
            return Role.get(self.receiver_id)
        else:
            return None

    def set_receiver(self, role):
        if role:
            self.receiver_id = role.id
        elif self.receiver_id:
            self.receiver_id = None
    receiver = property(get_receiver, set_receiver)

    def get_category(self):
        if self.category_id:
            return Category.get(self.category_id)
        else:
            return None

    def set_category(self, category):
        if category:
            self.category_id = category.id
        elif self.category_id:
            self.category_id = None
    category = property(get_category, set_category)

    def get_by_urlname(cls, url_name):
        objects = [x for x in cls.select() if x.url_name == url_name]
        if objects:
            return objects[0]
        raise KeyError()
    get_by_urlname = classmethod(get_by_urlname)


    def create_form(self, page_no, answers = {}):
        form = Form(enctype = "multipart/form-data", use_tokens = False)
        current_page = 0
        for field in self.fields:
            if field.type == 'newpage':
                current_page += 1
                if current_page > page_no:
                    break
                continue
            if current_page != page_no:
                continue
            if field.type == 'title':
                form.widgets.append(HtmlWidget(htmltext("<h3>%s</h3>" % field.label)))
                continue
            if field.type == 'subtitle':
                form.widgets.append(HtmlWidget(htmltext("<h4>%s</h4>" % field.label)))
                continue
            if field.type == 'comment':
                form.widgets.append(HtmlWidget(htmltext("<p>%s</p>" % field.label)))
                continue

            widget_class = widget_classes[field.type]

            kwargs = {'required': field.required}
            if field.type == 'item':
                kwargs['options'] = field.items or [(None, '---')]
                if field.show_as_radio:
                    widget_class = RadiobuttonsWidget
                    if len(kwargs['options']) > 2:
                        kwargs['delim'] = htmltext('<br />')
            kwargs['value'] = answers.get(field.id)
            if widget_extra_attributes.has_key(field.type):
                for k in widget_extra_attributes[field.type]:
                    if hasattr(field, k):
                        kwargs[k] = getattr(field, k)
            form.add(widget_class, "f%s" % field.id, title = field.label, **kwargs)

        if page_no != 0:
            form.add_submit("previous", _("Previous"))
        form.add_submit("submit", _("Next"))

        return form

    def get_data(self, form):
        d = {}
        for field in self.fields:
            widget = form.get_widget('f%s' % field.id)
            if widget:
                d[field.id] = widget.parse()
        return d

    def get_filled(self, session):
        completed = self.data_class().select(lambda x: x.user_id == session.user)
        if len(completed):
            return completed[0]

        completed = AnonymityLink.select(
                lambda x: x.formdata_type == 'consultation' and (
                            x.name_identifier == session.name_identifier or
                            x.key == session.anonymous_key))
        for c in completed:
            if c.formdata.consultation_id == self.id:
                return c.formdata

        raise KeyError

