import cStringIO
import os
import tempfile

from quixote import get_publisher, get_request, get_session
from quixote.http_request import Upload
from quixote.form import *
from quixote.html import htmltext, TemplateIO
from quixote.util import randbytes

QuixoteForm = Form

Widget.REQUIRED_ERROR = N_('required field')
get_error_orig = Widget.get_error

def get_i18n_error(self, request=None):
    error = get_error_orig(self, request)
    if error:
        return _(error)

def render(self):
    r = TemplateIO(html=True)
    classnames = '%s widget' % self.__class__.__name__
    if self.get_error():
        classnames += ' widget-with-error'
    r += htmltext('<div class="%s">') % classnames
    r += self.render_title(self.get_title())
    r += htmltext('<div class="content">')
    r += self.render_content()
    r += self.render_hint(self.get_hint())
    r += self.render_error(self.get_error())
    r += htmltext('</div>')
    r += htmltext('</div>')
    if self.render_br:
        r += htmltext('<br class="%s" />') % classnames
    r += htmltext('\n')
    return r.getvalue()

Widget.get_error = get_i18n_error
Widget.render = render

class Form(QuixoteForm):
    TOKEN_NOTICE = htmltext('<div class="errornotice">%s</div>' % _(
            "The form you have submitted is invalid.  Most "
            "likely it has been successfully submitted once "
            "already.  Please review the the form data "
            "and submit the form again."))
    ERROR_NOTICE = htmltext('<div class="errornotice">%s</div>' % _(
            "There were errors processing your form.  See below for details."))

    def add(self, widget_class, name, *args, **kwargs):
        if kwargs and not kwargs.has_key('render_br'):
            kwargs['render_br'] = False
        return QuixoteForm.add(self, widget_class, name, *args, **kwargs)

    def _render_submit_widgets(self):
        return htmltext('<div class="buttons">%s</div>' % QuixoteForm._render_submit_widgets(self))

    def _render_body(self):
        value = QuixoteForm._render_body(self)
        if self.has_errors():
            # if form has errors, focus and select the first widget on error
            value += htmltext("""<script type="text/javascript">
var all_obj = document.getElementsByTagName('DIV');
for (i=0; i<all_obj.length; i++) {
    if ( ( ' ' + all_obj[i].getAttribute("class") + ' ').toLowerCase().match(
                new RegExp ( '^.* widget-with-error .*$', 'g' ) ) ) {
        less_obj = all_obj[i].getElementsByTagName('INPUT');
        for (j=0; j<less_obj.length; j++) {
            less_obj[j].select();
            less_obj[j].focus();
            break;
        }
        break;
    }
}
</script>""")
        return value


class HtmlWidget:
    def __init__(self, string):
        self.string = string

    def render(self):
        return self.string

    def has_error(self, request):
        return False


class FakeFileWidget(CompositeWidget):
    """Widget that proposes a File Upload widget but that stores the file
    ondisk so it has a "readonly" mode where the filename is shown."""
    def __init__(self, name, value=None, preview = False, **kwargs):
        CompositeWidget.__init__(self, name, value, **kwargs)
        self.value = value
        if not preview:
            del kwargs['title']
            self.add(FileWidget, 'file', **kwargs)
        else:
            session = get_session()
            self.add(HiddenWidget, 'token')
            del kwargs['title']
            self.add(StringWidget, 'filename', **kwargs)
            if hasattr(get_request(), 'tempfiles'):
                token = get_request().tempfiles[name]
                wid = self.get_widget('token')
                get_request().form[wid.name] = token
                wid = self.get_widget('filename')
                get_request().form[wid.name] = session.tempfiles[token]['orig_filename']

    def render_content(self):
        r = TemplateIO(html=True)
        for widget in self.get_widgets():
            r += widget.render()
        return r.getvalue()

    def _parse(self, request):
        session = get_session()
        if self.get('file'):
            self.value = self.get('file')
            self.value.__class__ = PicklableUpload
            # and saves it
            token = randbytes(8)
            if not hasattr(get_request(), 'tempfiles'):
                get_request().tempfiles = {}
            get_request().tempfiles[self.name] = token
            dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
            if not os.path.exists(dirname):
                os.mkdir(dirname)
            filename = os.path.join(dirname, token)
            fd = open(filename, 'w')
            upload = self.value
            fd.write(upload.fp.read())
            fd.close()
            if not session.tempfiles:
                session.tempfiles = {}
            session.tempfiles[token] = {
                'orig_filename': upload.orig_filename,
                'base_filename': upload.base_filename,
                'content_type': upload.content_type,
                'charset': upload.charset
            }
        else:
            token = self.get('token')
            if session.tempfiles.has_key(token):
                self.value = PicklableUpload(session.tempfiles[token]['orig_filename'],
                    session.tempfiles[token]['content_type'],
                    session.tempfiles[token]['charset'])
                self.value.fp = tempfile.TemporaryFile()
                dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
                filename = os.path.join(dirname, token)
                self.value.fp.write(open(filename).read())
                self.value.fp.seek(0)

class PicklableUpload(Upload):
    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['fp']
        odict['data'] = self.fp.read()
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.fp = cStringIO.StringIO(self.data)
        del self.data

class EmailWidget(StringWidget):
    def _parse(self, request):
        StringWidget._parse(self, request)
        if self.value is not None:
            if not '@' in self.value: # ultra-basic, needs proper regex
                self.error = N_("must be a valid email address")

